# frozen_string_literal: true

module RenoteDac

  class PushPendingEmailsJob < Que::Job

    @priority = 10

    def run
      (destroy && return) unless obtained_lock?

      begin
        loop do
          if Que.worker_count.zero?
            raise 'Gracefully terminating job early...'
          end

          count = RenoteDac::Mailer::Client.new.process_batch

          puts "This job pushed #{count} pending emails."

          break if count.zero?
        end
      ensure
        release_lock
      end
    end

    private

    def obtained_lock?
      ActiveRecord::Base.connection.select_value('SELECT pg_try_advisory_lock(1);')
    end

    def release_lock
      ActiveRecord::Base.connection.select_value('SELECT pg_advisory_unlock(1);')
    end
  end

end